#import "TasksViewController.h"


@interface TasksViewController ()
- (NSPredicate*)filterRequest;
- (void)createFetchRequest;
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath;
@end

@implementation TasksViewController

@synthesize taskResults;

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.navigationItem.leftBarButtonItem = self.editButtonItem;
    
    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd 
                                                                               target:self 
                                                                               action:@selector(createTask)];
    self.navigationItem.rightBarButtonItem = addButton;
    
    
    [self createFetchRequest];
}

- (void)createFetchRequest {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    NSManagedObjectContext *managedObjectContext = appDelegate.managedObjectContext;
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    fetchRequest.entity = [NSEntityDescription entityForName:@"Task" 
                                      inManagedObjectContext:managedObjectContext];
    
    NSPredicate *filterRequest = self.filterRequest;
    if (filterRequest) {
        fetchRequest.predicate = filterRequest;
    }
    
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"timeStamp" ascending:NO];
    fetchRequest.sortDescriptors = [NSArray arrayWithObjects:sortDescriptor, nil];
    
    self.taskResults = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest 
                                                           managedObjectContext:managedObjectContext
                                                             sectionNameKeyPath:nil 
                                                                      cacheName:@"Master"];
    NSError *error = nil;
	if (![self.taskResults performFetch:&error]) {
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    abort();
	}
    self.taskResults.delegate = self;
}

- (void)createTask {
    
    NSString *nibName = [NSString stringWithFormat:@"TaskEditorViewController%@", prefixDevice];
    TaskEditorViewController *taskEditor = [[TaskEditorViewController alloc] 
                                            initWithNibName:nibName
                                            bundle:nil];
    [self.navigationController pushViewController:taskEditor animated:YES];
    
    
}

#pragma mark - Table view data source

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    
   
    
    Task *task = [self.taskResults objectAtIndexPath:indexPath];
    cell.textLabel.text = task.name;
    cell.detailTextLabel.text = [task.timeStamp description];
    /*cell.accessoryType = task.isCompleted.boolValue? UITableViewCellAccessoryCheckmark:UITableViewCellAccessoryNone;*/
    
    UIImage *image = [UIImage imageNamed:@"yesCM.png"];
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    CGRect frame = CGRectMake(0.0, 0.0, image.size.width, image.size.height);
    button.frame = frame;
    [button setBackgroundImage:image forState:UIControlStateNormal];
    UIColor *cellColor;
    cellColor =[[UIColor alloc] initWithRed:203.0f/255 green:115.0f/255  blue:58.0f/255 alpha:1];//;#15B4FF
    
    button.backgroundColor = [UIColor blackColor];
    cell.accessoryView = task.isCompleted.boolValue? button:nil;
    
    cell.textLabel.textColor = [UIColor whiteColor];
    cell.accessoryView.backgroundColor = cellColor;
    cell.detailTextLabel.textColor = [UIColor whiteColor];
  
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return self.taskResults.sections.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    id <NSFetchedResultsSectionInfo> sectionInfo = [[self.taskResults sections] objectAtIndex:section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    }
    
    [self configureCell:cell atIndexPath:indexPath];
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    /*UIAlertView *question = [[UIAlertView alloc] initWithTitle:@"Task" 
                                                       message:@"Выберите действие" 
                                                      delegate:self
                                             cancelButtonTitle:@"Cancel" 
                                             otherButtonTitles:@"Complete", @"Edit", nil];
    [question show];*/
    
    UIActionSheet *actSheet = [[UIActionSheet alloc] initWithTitle:@"Task" 
                                                          delegate:self 
                                                 cancelButtonTitle:@"Cancel" 
                                            destructiveButtonTitle:nil 
                                                 otherButtonTitles:@"Complete",@"edit",nil];
    [actSheet showInView:self.view];
}

-(void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
    NSIndexPath *selectedIndexPath = [self.tableView indexPathForSelectedRow];
    [self.tableView deselectRowAtIndexPath:selectedIndexPath animated:YES];
    
    if (buttonIndex == 0) {
        Task *task = [self.taskResults objectAtIndexPath:selectedIndexPath];
        task.isCompleted = [NSNumber numberWithBool:YES];
    } else if (buttonIndex == 1) {
        TaskEditorViewController *taskEditor = [[TaskEditorViewController alloc] 
                                                initWithNibName:@"TaskEditorViewController" 
                                                bundle:nil];
        taskEditor.currentTask = [self.taskResults objectAtIndexPath:selectedIndexPath];
        [self.navigationController pushViewController:taskEditor animated:YES];
        
    }
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    NSIndexPath *selectedIndexPath = [self.tableView indexPathForSelectedRow];
    [self.tableView deselectRowAtIndexPath:selectedIndexPath animated:YES];
    
    if (buttonIndex == 1) {
        Task *task = [self.taskResults objectAtIndexPath:selectedIndexPath];
        task.isCompleted = [NSNumber numberWithBool:YES];
    } else if (buttonIndex == 2) {
        TaskEditorViewController *taskEditor = [[TaskEditorViewController alloc] 
                                                initWithNibName:@"TaskEditorViewController" 
                                                bundle:nil];
        taskEditor.currentTask = [self.taskResults objectAtIndexPath:selectedIndexPath];
        [self.navigationController pushViewController:taskEditor animated:YES];
        
    }
}

- (void)tableView:(UITableView *)tableView 
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        NSManagedObjectContext *context = [self.taskResults managedObjectContext];
        [context deleteObject:[self.taskResults objectAtIndexPath:indexPath]];
        
        NSError *error = nil;
        if (![context save:&error]) {
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        }
    }    
}

#pragma mark - NSFetchedResultsControllerDelegate

- (NSPredicate*)filterRequest {
    if (![self.navigationItem.title isEqualToString:@"All"]) {
        return [NSPredicate predicateWithFormat:@"isCompleted==%i",
                [self.navigationItem.title isEqualToString:@"Completed"]];
    } else {
        return nil;
    }
}

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller 
  didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex 
     forChangeType:(NSFetchedResultsChangeType)type
{
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] 
                          withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] 
                          withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView = self.tableView;
    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] 
                             withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] 
                             withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] 
                             withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]
                             withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView endUpdates];
}




@end