//
//  UNActionPicker.m
//  iHomeMoney
//
//  Created by Eugene Romanishin on 23.01.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "UNActionPicker.h"

@implementation UNActionPicker

@synthesize picker;
@synthesize datePicker;
@synthesize items;
@synthesize delegate;
@synthesize customPickerDataSource;

- (id)initWithItems:(NSArray*)pickerItems title:(NSString*)title type:(PickerTypes)pickerType {
    self = [super initWithTitle:title delegate:nil cancelButtonTitle:nil destructiveButtonTitle:nil otherButtonTitles:nil];
    if (self) {
        self.items = pickerItems;
        
        CGRect pickerFrame = CGRectMake(0.0f, 40.0f, 320.0f, 485.0f);
        
        if (pickerType == DateType) {
            self.datePicker = [[UIDatePicker alloc] initWithFrame:pickerFrame];
            datePicker.datePickerMode = UIDatePickerModeDateAndTime;
            datePicker.date = [NSDate date];
            [self addSubview:datePicker];
        } else {
            self.picker = [[UIPickerView alloc] initWithFrame:pickerFrame];
            picker.showsSelectionIndicator = YES;
            if (pickerType == SimpleType) {
                picker.dataSource = self;
                picker.delegate = self;
            } else if (pickerType == CustomType) {
                self.customPickerDataSource = [[CustomPickerDataSource alloc] init];
                customPickerDataSource.items = self.items;
                
                picker.dataSource = customPickerDataSource;
                picker.delegate = customPickerDataSource;
            }
            [self addSubview:picker];
        }
        
    }
    return self;
}

- (void)showInView:(UIView *)view {
    [super showInView:view];
    [self setBounds:CGRectMake(0.0f, 0.0f, 320.0f, 485.0f)];
}

- (void)showFromTabBar:(UITabBar *)tabBar {
    [super showFromTabBar:tabBar];
    [self setBounds:CGRectMake(0.0f, 0.0f, 320.0f, 485.0f)];
}

- (void)setCloseButtonTitle:(NSString*)title color:(UIColor*)color {
    UISegmentedControl *doneButton = [[UISegmentedControl alloc] initWithItems:[NSArray arrayWithObject:title]];
    doneButton.momentary = YES; 
    doneButton.frame = CGRectMake(260.0f, 7.0f, 50.0f, 30.0f);
    doneButton.segmentedControlStyle = UISegmentedControlStyleBar;
    doneButton.tintColor = color;
    [doneButton addTarget:self action:@selector(dismissActionSheet) forControlEvents:UIControlEventValueChanged];
    [self addSubview:doneButton];
    

    UISegmentedControl *closeButton = [[UISegmentedControl alloc] initWithItems:[NSArray arrayWithObject:NSLocalizedString(@"Cancel", nil)]];
    closeButton.momentary = YES; 
    closeButton.frame = CGRectMake(10.0f, 7.0f, 70.0f, 30.0f);
    closeButton.segmentedControlStyle = UISegmentedControlStyleBar;
    closeButton.tintColor = color;
    [closeButton addTarget:self action:@selector(closeActionShee) forControlEvents:UIControlEventValueChanged];
    [self addSubview:closeButton];
}

- (void)closeActionShee {
    [self dismissWithClickedButtonIndex:0 animated:YES];
}

- (void)dismissActionSheet {
    if ([delegate respondsToSelector:@selector(didSelectItem:)]) {
        id selectedObject = nil;
        if (picker) {
            selectedObject = [items objectAtIndex:[picker selectedRowInComponent:0]];
        } else {
            selectedObject = datePicker.date;
        }
        
        [delegate performSelector:@selector(didSelectItem:) withObject:selectedObject];
    }
    
    [self closeActionShee];
}

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)thePickerView 
{
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)thePickerView 
numberOfRowsInComponent:(NSInteger)component 
{
    return items.count;
}

- (NSString *)pickerView:(UIPickerView *)thePickerView 
             titleForRow:(NSInteger)row 
            forComponent:(NSInteger)component 
{
    NSString *title = nil;
    
    id item = [items objectAtIndex:row];
    if ([item isKindOfClass:[NSDictionary class]]) {
        title = [item objectForKey:@"shortname"];
    } else {
        title = [item name];
    }
    
    return title;
}

@end
